#ifndef AlgorithmPerformancesPredictor_HPP
#define AlgorithmPerformancesPredictor_HPP


#include <DoQuBook/DoQuBook_global.hpp>

#include <DoQuBook/Page.hpp>
#include <DoQuBook/Region.hpp>
#include <DoQuBook/Config.hpp>

#include <DoQuBook/AlgorithmOnTask.hpp>

namespace  DoQuBook {
class PluginLoader;
}

class DoQuBookSHARED_EXPORT AlgorithmPerformancesPredictor  : public DoQuBook::AlgorithmOnTask
{
  public :
    class TrainingConfiguration {

      public:
        TrainingConfiguration () : useRepo (false), _loader (NULL) {}

        // If set to true, use features  stored on a repository
        // If set to false use available plugins only
        TrainingConfiguration (bool selectFeatOnARepo) : useRepo (selectFeatOnARepo) {}

        QList<DoQuBook::PagePtr> traingingSet() const {return _traingingSet;}
        void setTraingingSet(const QList<DoQuBook::PagePtr>& traingingSet){_traingingSet = traingingSet;}

        QList<DoQuBook::RepositoryPtr> repos() const{return _repos;}
        void setRepos(const QList<DoQuBook::RepositoryPtr>& repos){_repos = repos;}

        QList<QString> measurers() const {return _measurers;}
        void setMeasurers(const QList<QString>& measurers) {_measurers = measurers;}

        /*QList<QString> algorithms() const{return _algorithms;}
        void setAlgorithms(const QList<QString>& algorithms) {_algorithms = algorithms;}
        */

        PluginBasePtr groundTruthBencher() const {return _groundTruthBencher;}
        void setGroundTruthBencher(const PluginBasePtr& groundTruthBencher){_groundTruthBencher = groundTruthBencher;}

        DoQuBook::PluginLoader* getLoader() const {return _loader;}
        void setLoader(DoQuBook::PluginLoader* value) {_loader = value;}


      private :
        QList<DoQuBook::RepositoryPtr> _repos;
        QList<DoQuBook::PagePtr> _traingingSet;
        bool useRepo; // if false use Possible Plugins.

        QList<QString> _measurers;
        //QList<QString> _algorithms;
        PluginBasePtr _groundTruthBencher;
        DoQuBook::PluginLoader *_loader;


        // TODO : setters and getters;
        /*QList<PluginBasePtr> _algorithms;
        GroundTruthProviderPtr provider;
        QList<PluginBasePtr> _groundTruthBencher; // fscore or other comparator plugins.

        QList<PluginBasePtr> _measurers;*/
        // need to save the used measurer plugins in a conf file.
    };

  public:
    AlgorithmPerformancesPredictor() {}
    ~AlgorithmPerformancesPredictor() {}

    // --- To be implemented //
    virtual QString organisationName () = 0;
    virtual QString description () = 0;
    virtual QString algorithmName () = 0;
    virtual QString version() = 0;
    virtual double predict(const QImage & image, const QVariantMap & params = QVariantMap() ) = 0;
    virtual void train (const TrainingConfiguration &conf) = 0;
    virtual QString predictedAlgorithmId() = 0;
    // --- //

    virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
      QList<DoQuBook::ExecutorParameterPtr> params;
      return params;
    }

    virtual QString pluginType () {
      return "Binarizer";
    }

    virtual QString returnType() {
      return "AlgorithmPerformancesPredictor";
    }

    virtual QList<DoQuBook::ExecutorParameterPtr> methodParameters() {
      QList<DoQuBook::ExecutorParameterPtr> params;
      params << DoQuBook::ExecutorParameter::create ("image", "Image", true);
      return params;
    }

    virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) {
      qDebug() << "Binarizing !!! ";
      QImage image = pluginMethodParams["image"].value<QImage>();
      return predict (image, params);
    }




};


typedef std_shared_ptr<AlgorithmPerformancesPredictor>::type AlgorithmPerformancesPredictorPtr;

#define AlgorithmPerformancesPredictor_iid "fr.labri.doqubook.AlgorithmPerformancesPredictor"

Q_DECLARE_INTERFACE (AlgorithmPerformancesPredictor, AlgorithmPerformancesPredictor_iid)

#endif // AlgorithmPerformancesPredictor_HPP
