/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */#ifndef BOOKIMPORTERREGEXP_HPP
#define BOOKIMPORTERREGEXP_HPP

#include <DoQuBook/BookImporter.hpp>
#include <QDir>
#include <QMap>
#include <DoQuBook/DoQuBook_global.hpp>

namespace DoQuBook
{


    /*!
      * Tries to import a book from a directory with
      * a given simplified regular expression :
        \code
        QString dirPath = "/Users/zoolonly/Pictures/Scans_Documents/BD/documents_ancien/72";

        QString extention = "jpg";
        QString separator = "_";

        QString regExp = "bookname"+separator+"pagenumber";

        BookImporterRegExp importer(filePath, regExp, extention);
        BookPtr book = importer.import();

        \endcode
      * In the previous example, filenames corresponds to BMP72_0009.jpg
      * BMP72 is the book name, 0009 the page number and .jpg the extention of the file.
      * The imported book will have for name BMP72 and the page will have a name equal to BMP720009.
      */
    class DoQuBookSHARED_EXPORT BookImporterRegExp : public BookImporter
    {
        public:
            BookImporterRegExp (const QString & dirPath,
                                const QString & regExp,
                                const QString & extention);
            virtual BookPtr import();

        private :
            QMap<QString, QString> regExpTranslationMap;
            QDir bookDir;
            QString ext;
            QString regExp;

    };

}

#endif // BOOKIMPORTERREGEXP_HPP
