/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef BOOKSERVICESERVER_HPP
#define BOOKSERVICESERVER_HPP


#include <QNetworkReply>

#include <QObject>
#include <DoQuBook/SimpleMarshallable.hpp>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/SimpleMarshallableWithId.hpp>
#include <DoQuBook/RESTObjectManager.hpp>
#include <DoQuBook/RESTImageManager.hpp>
#include <DoQuBook/Boolean.hpp>

#include <DoQuBook/DoQuBookWebService.hpp>
#include <DoQuBook/BookService.hpp>
#include <DoQuBook/Intlist.hpp>


namespace DoQuBook
{

    class ImageService;


    class Point;
    typedef QSharedPointer<Point> PointPtr;


    class DBWebBookService ;
    typedef QSharedPointer<DBWebBookService> DBWebBookServicePtr;

    /**
      * \todo Extract Image services from this class
      * \todo It should be easy to provide generic services and make a kind of
      * RESTEasy C++ Framework based on Qt Framework and reflexion.
      */
    class DoQuBookSHARED_EXPORT DBWebBookService : public BookService
    {
            Q_OBJECT

        public:

            QString server_adress;
            QString rest_service_url;
            QString book_ressource;
            QString image_ressource;

            DBWebBookService (const QString & serverAdress);

            QList<int> getAllBooks ();

            BookPtr get (const int & id);
            void del (const int & name);
            void del (BookPtr book);
            void update (BookPtr book);


            bool put (BookPtr b);

            bool uploadImage (PagePtr page);
            QImage getImage (PagePtr page, int w = -1);
            QImage getImage (const int & id, int w = -1);
            bool imageExistsOnServer (PagePtr page, int w = -1);
            bool imageExistsOnServer (const int & id, int w = -1);

            bool isAvailable();

        signals:
            void progress (int, int);

        protected :
            void injectImageServiceToPages (const BookPtr & book);

        private :
            void putMarshallable (MarshallablePtr m);
            //void replaceNewIdsPages(QList<SimpleMarshallableWithIdPtr> oldBook, QList<SimpleMarshallableWithIdPtr> onDBBook);
            AnnotationPtr findAnnotationEquals (PagePtr page, AnnotationPtr autre);
            AnnotatedPolygonPtr findPolyEquals (AnnotationLocalPtr ann, AnnotatedPolygonPtr poly);
            PointPtr findPointEquals (AnnotatedPolygonPtr poly, PointPtr point);

        private :
            RESTObjectManager<Book> m_bookManager;
            RESTObjectManager<Intlist> m_bookListManager;
            RESTImageManager m_pageImageManager;
            RESTObjectManager<Boolean> m_existsImageManager;
    };
}

#endif // BOOKSERVICESERVER_HPP
