/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */#ifndef DEFECT_HPP
#define DEFECT_HPP

#include <QString>
#include <QVariant>

namespace DoQuBook
{

    class Defects;

    class Defect
    {
            friend class Defects;
        protected :
            Defect (const QString & name, QVariant::Type type);
        public :
            const QString getName() const {
                return _name;
            }

            QVariant::Type getType() const {
                return _type;
            }
        protected :
            QString _name;
            /// Can be an int or a String
            QVariant::Type _type;
    };


    /**
      * \warning Every new Defect added here must be registered in DefectsServices MANUALLY
      */
    class Defects
    {
        public :
            /* Global Boolean Values */
            static const Defect Global_IsWhitePage;
            /* Global Int Values */
            static const Defect Global_OverAllQuality_Perception;
            static const Defect Global_TextSizeVariation_Perception;
            static const Defect Global_QuantityText_Perception;
            static const Defect Global_QuantityImages_Perception;
            static const Defect Global_QuantityBleedThrough_Perception;
            static const Defect Global_QuantitySpots_Perception;
            static const Defect Global_QualityText_Perception;
            /* Local Int Values Annotations */
            static const Defect Local_BleedThrough_Perception;
            static const Defect Local_Spot_Perception;
            /* Global Registration parameters */
            static const Defect Global_Registration_X;
            static const Defect Global_Registration_Y;
            static const Defect Global_Registration_Teta;
            /* Recto - Verso */
            static const Defect Verso;
            /* Global Trinarization */
            static const Defect    Global_BleedThroughGrayScale_Ink;
            static const Defect    Global_BleedThroughGrayScale_Background;
            /* Local Trinarization */
            static const Defect    Local_BleedThroughGrayScale_Ink;
            static const Defect    Local_BleedThroughGrayScale_Background;

            /* Compressions */
            static const Defect    Global_JPEG_Compression;

    };

}

#endif // DEFECT_HPP
