/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef DOQUBOOKAPP_HPP
#define DOQUBOOKAPP_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <QApplication>
#include <QObject>
#include <QMap>
#include <DoQuBook/Page.hpp>
#include <DoQuBook/Repository.hpp>

namespace DoQuBook
{

    class DoQuBookSHARED_EXPORT DoQuBookApp : public QObject
    {
            Q_OBJECT
        public:
            explicit DoQuBookApp (QCoreApplication & app,
                                  const QString & orgName,
                                  const QString & appName,
                                  const QString & exeName = "");

            class ProgramParameter
            {
                public :
                    ProgramParameter (QString name,
                                      bool required,
                                      QString description)
                        : _name (name), _required (required), _description (description)
                    {}

                    QString _name;
                    bool _required;
                    QString _description;
            };

        signals:
            void done();

        public slots:
            int exec ();
            int execConsoleApp ();
            virtual void runSlot ();
            virtual void installAsDaemon();
            virtual void uninstallDaemon();
            void showHelpMessage();

        protected :
            virtual void runApp (QMap<QString, QString> args) = 0;
            /// key = options, value = descriptions;
            virtual QList<ProgramParameter> paramsAndDescriptions();

            virtual DoQuBook::PagePtr getPageFromParameters (DoQuBook::RepositoryPtr repo = DoQuBook::RepositoryPtr() );
            virtual DoQuBook::RepositoryPtr getRepoFromParameters();

        private :
            void initParams();


        protected :
            QCoreApplication & _app;
            const QString & _orgName;
            const QString & _appName;
            QString _exeName;
            QMap<QString, QString> _params;
    };
}

#endif // DOQUBOOKAPP_HPP
