#ifndef EXECUTOR_HPP
#define EXECUTOR_HPP

#include <QString>
#include <DoQuBook/DoQuBook_global.hpp>
#include <QSharedPointer>
#include <QVariant>
#include <DoQuBook/SimpleMarshallable.hpp>

namespace DoQuBook
{

    class Executor;
    typedef QSharedPointer<Executor> ExecutorPtr;

    class ExecutorParameter;
    typedef QSharedPointer<ExecutorParameter> ExecutorParameterPtr;

    class DoQuBookSHARED_EXPORT ExecutorParameter : public SimpleMarshallable
    {
            Q_OBJECT

        public :
            ExecutorParameter() :
                m_id (-1), m_required (false)
            {}
            virtual ~ExecutorParameter() {}

            MARSHALLABLE_CREATE_FONCTIONS (ExecutorParameter)

            static ExecutorParameterPtr create (const QString & name,
                                                const QString & type,
                                                bool required);

            int getId() const {
                return m_id;
            }
            QString getName() const {
                return m_name;
            }

            QString getParameterType() const {
                return m_parameterType;
            }

            bool getRequired() const {
                return m_required;
            }

        public slots:
            void setId (int arg) {
                m_id = arg;
            }
            void setName (QString arg) {
                m_name = arg;
            }
            void setParameterType (QString arg) {
                m_parameterType = arg;
            }
            void setRequired (bool arg) {
                m_required = arg;
            }

        private :
            Q_PROPERTY (int id READ getId WRITE setId)
            int m_id;

            Q_PROPERTY (QString name READ getName WRITE setName)
            QString m_name;

            Q_PROPERTY (QString parameterType READ getParameterType WRITE setParameterType)
            QString m_parameterType;

            Q_PROPERTY (bool required READ getRequired WRITE setRequired)
            bool m_required;
    };


    class DoQuBookSHARED_EXPORT Executor :  public SimpleMarshallable
    {
            Q_OBJECT

            Q_PROPERTY (QString id READ getId WRITE setId)

            Q_PROPERTY (QString description READ getDescription WRITE setDescription)

            Q_PROPERTY (QString pluginType READ getPluginType WRITE setPluginType)

            Q_PROPERTY (QString returnType READ getReturnType WRITE setReturnType)

            Q_PROPERTY (QList<ExecutorParameterPtr> params READ getParams WRITE setParams)

        public:

            MARSHALLABLE_CREATE_FONCTIONS (Executor)

            MARSHALLABLE_MLIST (QList, ExecutorParameter, params)
            MARSHALLABLE_ACTIV_1LIST (params, m_params)

            Executor();

            QString getId() const {
                return m_id;
            }


            QString getDescription() const {
                return m_description;
            }

            QString getReturnType () const {
                return m_returnType ;
            }

            QList<ExecutorParameterPtr> getParams() const {
                return m_params;
            }

            QString getPluginType() const {
                return m_pluginType;
            }

        public slots:

            void setId (QString arg) {
                m_id = arg;
            }

            void setDescription (QString arg) {
                m_description = arg;
            }

            void setReturnType (QString arg) {
                m_returnType  = arg;
            }

            void addParam (const QString & name, const QString & parameterType, bool required = false);

            void addParam (ExecutorParameterPtr param);

            void setParams (QList<ExecutorParameterPtr> arg) {
                m_params = arg;
            }

            void setPluginType (QString arg) {
                m_pluginType = arg;
            }

        private :
            QString m_id;


            QString m_description;


            QString m_returnType ;

            QList<ExecutorParameterPtr> m_params;
            QString m_pluginType;
    };

}

#endif // EXECUTOR_HPP
