#ifndef IMAGECOMPARATOR_HPP
#define IMAGECOMPARATOR_HPP


#include <QVariant>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/AlgorithmOnTask.hpp>

class DoQuBookSHARED_EXPORT ImageComparator : public DoQuBook::AlgorithmOnTask
{
    public :
        ImageComparator () {}
        virtual ~ImageComparator () {}

        // --- To be implemented //
        virtual QString organisationName () = 0;
        virtual QString description () = 0;
        virtual QString algorithmName () = 0;
        virtual QString version() = 0;
        virtual double compare (const QImage & image1, const QImage & image2) = 0;
        // --- //

        virtual QString pluginType () {
            return "ImageComparator";
        }
        virtual QString returnType() {
            return "Numeric";
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> methodParameters() {
            QList<DoQuBook::ExecutorParameterPtr> params;
            params << DoQuBook::ExecutorParameter::create ("image1", "Image", true);
            params << DoQuBook::ExecutorParameter::create ("image2", "Image", true);
            return params;
        }

        virtual QVariant compute (const QVariantMap & pluginMethodParams, const QVariantMap & params = QVariantMap() ) {
            QImage image1 = pluginMethodParams["image1"].value<QImage>();
            QImage image2 = pluginMethodParams["image2"].value<QImage>();
            return compare (image1, image2);
        }

        virtual QList<DoQuBook::ExecutorParameterPtr> parameters () {
            QList<DoQuBook::ExecutorParameterPtr> params;
            return params;
        }

};


typedef std_shared_ptr<ImageComparator>::type ImageComparatorPtr;
#define ImageComparator_iid "fr.labri.doqubook.ImageComparator"

Q_DECLARE_INTERFACE (ImageComparator, ImageComparator_iid)


#endif // IMAGECOMPARATOR_HPP
