#ifndef PAGESERVICE_HPP
#define PAGESERVICE_HPP

#include <QSharedPointer>
#include <DoQuBook/Page.hpp>
#include <DoQuBook/CRUDService.hpp>
#include <QFileInfo>
#include <DoQuBook/ListRestFileAttachment.hpp>
#include <DoQuBook/Future.hpp>
#include <DoQuBook/PageEvent.hpp>

namespace DoQuBook
{



  class PageRESTResponseManager : public AsyncWebClientQt<Page> {
      Q_OBJECT
  public :
      virtual void onReceived(QSharedPointer<Page> m) {
          emit pageReceived(m);
      }
    signals :
      void pageReceived(QSharedPointer<Page> m);
  };

    class PageService;
    typedef QSharedPointer<PageService> PageServicePtr;

    class PageService : virtual public CRUDService<Page, int>
    {
        public:
            PageService();
            virtual ~PageService();

            virtual meta_iterator<PagePtr>::IteratorPtr getAll() = 0;
            virtual ListRestFileAttachmentPtr getAllAttachments (const PagePtr p) = 0;

            virtual void addAttachment (const PagePtr page, const QFileInfo & file) = 0;

            virtual meta_iterator<PagePtr>::IteratorPtr getByName (const QString & name) = 0;
            virtual PagePtr findByName (const QString & name) = 0;

            virtual QStringList existingTags() = 0;
            virtual meta_iterator<PagePtr>::IteratorPtr findByTag (const QString & tag) = 0;
            virtual void addTag(const PagePtr &p, const QString &tag) = 0;
            virtual void removeTag(const PagePtr &p, const QString &tag) = 0;
            virtual PageEvent* onPageTaggedEvent(const QString &tag, QList< Functor<Page>* > functors = QList< Functor<Page>* >()) = 0;
#ifdef HAVE_CPP11
            virtual void onPageTaggedEvent(const QString &tag, std::function<void (const PagePtr&)> func) = 0;
#endif

            virtual Future<Page>* onPageAdded (Functor<Page> *functor) = 0;

      public :
            static const QString TAG_BLURRED;
            static const QString TAG_WHITE_PAGE;
            static const QString TAG_BLEEDTHROUGH;
            static const QString TAG_BOOK_COVER;
            static const QString TAG_FOLDED;
            static const QString TAG_REAL_IMAGE;
            static const QString TAG_SYNTHETIC_IMAGE;

    };

}
#endif // PAGESERVICE_HPP
