#ifndef PERFORMANCEMETRICSERVICE_HPP
#define PERFORMANCEMETRICSERVICE_HPP

#include <DoQuBook/DoQuBook_global.hpp>
#include <QSharedPointer>
#include <DoQuBook/CRUDService.hpp>
#include <DoQuBook/SimpleMarshallable.hpp>
#include <DoQuBook/Page.hpp>

namespace DoQuBook
{

    class PerformanceMetricService;
    typedef QSharedPointer<PerformanceMetricService> PerformanceMetricServicePtr;



    class PerformanceMetric;
    typedef QSharedPointer<PerformanceMetric> PerformanceMetricPtr;

    class DoQuBookSHARED_EXPORT  PerformanceMetric : public SimpleMarshallable
    {

            Q_OBJECT

        public :

            MARSHALLABLE_CREATE_FONCTIONS (PerformanceMetric)

            explicit PerformanceMetric (QObject * parent = NULL);
            virtual ~PerformanceMetric();

            int getId() const;
            QString getAlgorithmName() const;
            QString getMetricName() const;
            double getMetricValue() const;

        public slots:
            void setId (int arg);
            void setAlgorithmName (QString arg);
            void setMetricName (QString arg);
            void setMetricValue (double arg);

        signals:
            void idChanged (int arg);
            void algorithmNameChanged (QString arg);
            void metricNameChanged (QString arg);
            void metricValueChanged (double arg);

        protected :
            Q_PROPERTY (int id READ getId WRITE setId NOTIFY idChanged)
            Q_PROPERTY (QString algorithmName READ getAlgorithmName WRITE setAlgorithmName NOTIFY algorithmNameChanged)
            Q_PROPERTY (QString metricName READ getMetricName WRITE setMetricName NOTIFY metricNameChanged)
            Q_PROPERTY (double metricValue READ getMetricValue  WRITE setMetricValue NOTIFY metricValueChanged)


        private :
            int m_id;
            QString m_algorithmName;
            QString m_metricName;
            double m_metricValue;
    };

    class DoQuBookSHARED_EXPORT PerformanceMetricService : virtual public CRUDService<PerformanceMetric, int>
    {
        public:
            PerformanceMetricService() {}
            virtual ~PerformanceMetricService() {}

            virtual void addToPage (QSharedPointer<PerformanceMetric> &obj, PagePtr page) = 0;
            virtual meta_iterator<PagePtr>::IteratorPtr getPagesByMetricName (const QString & metricName) = 0;
            virtual PerformanceMetricPtr getByPageMetricNameAndAlgorithmName (PagePtr page, QString metricName, QString algorithmName) = 0;

    };

}

#endif // PERFORMANCEMETRICSERVICE_HPP
