/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef REPOSITORY_HPP
#define REPOSITORY_HPP

#include <QString>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/SimpleMarshallable.hpp>

#include <DoQuBook/BookService.hpp>
#include <DoQuBook/ImageService.hpp>
#include <DoQuBook/PageCollectionService.hpp>

#include <DoQuBook/TaskService.hpp>
#include <DoQuBook/TaskImageService.hpp>

#include <DoQuBook/ExecutorService.hpp>
#include <DoQuBook/PageService.hpp>

#include <DoQuBook/DaemonService.hpp>

#include <DoQuBook/GroundTruthService.hpp>
#include <DoQuBook/PerformanceMetricService.hpp>
#include <DoQuBook/FeaturesService.hpp>

#include <DoQuBook/PredictionModelResultService.hpp>
namespace DoQuBook
{

    class Repository;
    typedef QSharedPointer<Repository> RepositoryPtr;

    class DoQuBookSHARED_EXPORT Repository : public SimpleMarshallable
    {
            Q_OBJECT

            Q_PROPERTY (QString name READ getName WRITE setName)

        public:

            //MARSHALLABLE_CREATE_FONCTIONS (Repository)

            Repository();

            virtual PageServicePtr getPageService() = 0;
            virtual BookServicePtr getBookService() = 0;
            virtual ImageServicePtr getImamgeService() = 0;
            virtual PageCollectionServicePtr getPageCollectionService() = 0;
            virtual TaskServicePtr getTaskService() = 0;
            virtual TaskImageServicePtr getTaskImageService() = 0;
            virtual TempImageServicePtr getTempImageService() = 0;
            virtual ExecutorServicePtr getExecutorService() = 0;
            virtual DaemonServicePtr getDaemonService() = 0;
            virtual GroundTruthServicePtr getGroundTruthService() = 0;
            virtual PerformanceMetricServicePtr getPerformanceMetricService() = 0;
            virtual FeatureServicePtr getFeatureService() = 0;
            virtual PredictionModelResultServicePtr getPredictionModelResultService() = 0;

            QString getName();

            void setName (const QString & getName);

            void save (const QString & fileName);

            virtual void setParam (QVariant v) = 0;

            virtual QVariant getParam() = 0;

            /*!
              * This enum can be used in a GUI in order to create a widget
              *  that best describe the provider parameter.
              * For example :
              *  - an URL would be of type LineEdit.
              *  - a directory woold be of type FileDialog.
              */
            enum RepositoryParamWidgetType { LineEdit, FileDialog };

            virtual RepositoryParamWidgetType widgetType() = 0;

            //    void setProvider(BookProviderPtr provider);
            //    BookProviderPtr getProvider();

        private :
            QString _name;
    };


}
#endif // REPOSITORY_HPP
