#ifndef STATSSERVICE_HPP
#define STATSSERVICE_HPP

#include <QObject>
#include <DoQuBook/SimpleMarshallable.hpp>

namespace DoQuBook
{



    class ListDoubleRestElement;
    typedef QSharedPointer<ListDoubleRestElement> ListDoubleRestElementPtr;

    class DoQuBookSHARED_EXPORT ListDoubleRestElement : public SimpleMarshallable
    {
            Q_OBJECT
            Q_PROPERTY (QList<double> values READ getValues WRITE setValues)

        public :

            MARSHALLABLE_CREATE_FONCTIONS (ListDoubleRestElement)

            MARSHALLABLE_QTLIST (QList, double, values)
            MARSHALLABLE_ACTIV_1LIST (values, _values)

            ListDoubleRestElement()
            {}
            QList<double>  getValues() {
                return _values;
            }
            void setValues (QList<double> pValues) {
                _values = pValues;
            }

        private :
            QList<double> _values;

    };


    class ListListDoubleRestElement;
    typedef QSharedPointer<ListListDoubleRestElement> ListListDoubleRestElementPtr;

    class DoQuBookSHARED_EXPORT ListListDoubleRestElement : public SimpleMarshallable
    {
            Q_OBJECT
            Q_PROPERTY (QList<ListDoubleRestElementPtr> lists READ getLists WRITE setLists)

        public :

            MARSHALLABLE_CREATE_FONCTIONS (ListListDoubleRestElement)

            MARSHALLABLE_MLIST (QList, ListDoubleRestElement, lists)
            MARSHALLABLE_ACTIV_1LIST (lists, _lists)

            ListListDoubleRestElement()
            {}
            QList<ListDoubleRestElementPtr>  getLists() {
                return _lists;
            }
            void setLists (QList<ListDoubleRestElementPtr> pValues) {
                _lists = pValues;
            }

            void addList (ListDoubleRestElementPtr l) {
                _lists.append (l);
            }
        private :
            QList<ListDoubleRestElementPtr> _lists;

    };


    class IrrListResult;
    typedef QSharedPointer<IrrListResult> IrrListResultPtr;

    class DoQuBookSHARED_EXPORT IrrListResult : public SimpleMarshallable
    {
            Q_OBJECT
            /**
             * a character QString describing the method applied for the computation of interrater reliability.
             */
            Q_PROPERTY (QString method READ getMethod WRITE setMethod)
            /**
             *  the number of subjects examined.
             */
            Q_PROPERTY (int subjects READ getSubjects WRITE setSubjects)
            /**
             * the number of raters.
             */
            Q_PROPERTY (QString raters READ getRaters WRITE setRaters)

            /**
             * a character QString specifying the name of the coefﬁcient.
             */
            Q_PROPERTY (QString name READ getName WRITE setName)


            Q_PROPERTY (double value READ getValue WRITE setValue)

        public :

            MARSHALLABLE_CREATE_FONCTIONS (IrrListResult)


            QString getMethod() const {
                return m_method;
            }

            int getSubjects() const {
                return m_subjects;
            }

            QString getRaters() const {
                return m_raters;
            }

            QString getName() const {
                return m_name;
            }

            double getValue() const {
                return m_value;
            }

            void show() {
                qDebug() << m_method;
                qDebug() << "Test name :" << m_name;
                qDebug() << "Number of subjects : " << m_subjects;
                qDebug() << "Number of raters : " << m_raters;
                qDebug() << "Value : " << m_value;
            }

        public slots:
            void setMethod (QString arg) {
                m_method = arg;
            }

            void setSubjects (int arg) {
                m_subjects = arg;
            }

            void setRaters (QString arg) {
                m_raters = arg;
            }

            void setName (QString arg) {
                m_name = arg;
            }

            void setValue (double arg) {
                m_value = arg;
            }
        private :
            IrrListResult()
            {}

        private :
            QString m_method;
            int m_subjects;
            QString m_raters;
            QString m_name;
            double m_value;
    };


    class StatsService : public QObject
    {
            Q_OBJECT
        public:
            explicit StatsService (QString serverAdress, QObject * parent = 0);

            IrrListResultPtr kappaTest (QList<double> obs1, QList<double> obs2);

        signals:

        public slots:


        private :
            QString server_adress;
            QString rest_service_url;
            QString stats_ressource;
    };

}

#endif // STATSSERVICE_HPP
