#ifndef WEBCLIENT_HPP
#define WEBCLIENT_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <QObject>
#include <QByteArray>
#include <QUrl>
#include <QNetworkRequest>


#include <QtNetwork>
#include <QString>
#include <iostream>

#ifdef HAVE_CPP11
#include <future>
#include <thread>
#endif



#include <DoQuBook/MarshallableFormater.hpp>


namespace DoQuBook
{

    class DoQuBookSHARED_EXPORT WebClient : public QObject
    {
            Q_OBJECT
        public:
            explicit WebClient (QObject * parent = 0);
            enum Method {
                GET,
                POST,
                PUT,
                DEL
            };
            enum ContentType {
                JSON,
                XML,
                JPEG_MULTIPART,
                PNG_MULTIPART,
                TIFF_MULTIPART,
                XML_MULTIPART,
                ZIP_MULTIPART
            };

            void setContentType (ContentType t);

            QByteArray sendSynchronous  (const QUrl & url, Method m, const QByteArray & content );
            QByteArray sendSynchronous (const QUrl & url, Method m, const QByteArray & content, const QString & fileName );
            QByteArray postJpegFile  (const QUrl & url, const QByteArray & content, const QString & fileName );
            bool isAvailable (const QUrl & url);

            QByteArray gUncompress (const QByteArray & data);
            QByteArray gCompress (const QByteArray & data);


        signals:

        public slots:

        private :
            QString contentTypeToString();
            QByteArray boundary();
            void prepareRequestForMultiPart (QNetworkRequest & request, const QByteArray & boundary, const QByteArray & data);
            QByteArray createFormDataBytes (const QByteArray & bytes, const QByteArray & boundary, const QString & fileName );



        private :
            ContentType m_contentType;
    };

#include <QSharedPointer>

//http://qt-project.org/wiki/Threads_Events_QObjects
    class HttpCallManager  : public QObject
    {

            Q_OBJECT

        public :
            HttpCallManager (const QString & url) :
                m (NULL)
                , _url (url)
            {}

            void initAll() {
                m = new QNetworkAccessManager (this);
                connect (m, SIGNAL (finished (QNetworkReply *) ), this, SLOT (replyFinished (QNetworkReply *) ) );
            }

            ~HttpCallManager () {
                //m->deleteLater();
            }

            void run();

        signals :
            void contentReady (QByteArray content);

        public slots:
            void replyFinished (QNetworkReply * r) {
                if (r->error() != QNetworkReply::NoError) {
                    error (r->error() );
                }
                else {

                    QByteArray array = r->readAll();
                    if (array.size() > 0) {
                        emit contentReady (array);
                    }
                    else {
                        qCritical() << "HttpCallManager response is empty";
                    }
                }
                run();

            }

            void readyRead() {

            }

            void error (QNetworkReply::NetworkError /*error*/) {
                std::cout << "error" << std::endl;
            }


        private :
            QNetworkAccessManager * m;
            QString _url;
    };


    template <class T>
    class Unmarshall {
      protected :
        virtual void unmarshall (QNetworkReply *t) {
            QSharedPointer<T> m = m_formater.unMarshall(t->readAll());
            onReceived(m);
        }
        virtual void onReceived(QSharedPointer<T> m) = 0;
      private :
        MarshallableFormaterJSon<T> m_formater;
    };

    class AsyncWebClientQtBase : public QObject {
      Q_OBJECT
      public :
        void get(const QString &url) {
          QNetworkRequest  requete (url);
          requete.setHeader (QNetworkRequest::ContentTypeHeader, "application/json");
          requete.setRawHeader ("Accept", "application/json");
          connect(&_m, SIGNAL(finished(QNetworkReply*)), this, SLOT(process(QNetworkReply*)));
          _m.get (requete);
        }

      private slots :
        virtual void process (QNetworkReply *r) = 0;
      private :
        QNetworkAccessManager _m;
    };

    template <class T>
    class AsyncWebClientQt : public AsyncWebClientQtBase, public Unmarshall<T> {
      public :
        virtual void process (QNetworkReply *r) {
          Unmarshall<T>::unmarshall(r);
        }

        virtual void onReceived(QSharedPointer<T> m) = 0;
    };



#ifdef HAVE_CPP11
     class AsyncWebClientHelper {

       public :
         std::future<QByteArray> get(const QString &url, std::function<void (const QByteArray&)> onSuccess);

       private :
         std::promise<QByteArray> p;
         std::thread t;

     };
#endif

}

#endif // WEBCLIENT_HPP
