#include <DoQuBook/PluginLoader.hpp>
#include <DoQuBook/DoQuBookApp.hpp>
#include <DoQuBook/ApplicationContext.hpp>

using namespace DoQuBook;

QString orgName ("your_organization");
/// Must be the name of the target in CMake
QString appName ("@AppName@");

class @AppName@ : public DoQuBook::DoQuBookApp
{
public :
    @AppName@(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)

    {}

    QList<ProgramParameter> paramsAndDescriptions()
    {
        QList<ProgramParameter> params;
        //ProgramParameter param1 ("param1Name", false, "Param1 Description");
        //params.append(param1);
        params << DoQuBook::DoQuBookApp::paramsAndDescriptions();
        return params;
    }

protected :
    virtual void runApp (QMap<QString, QString> /*args*/)
    {
        // Get the needed services
        DoQuBook::RepositoryPtr repo = DoQuBook::ApplicationContext::instance()->getDefaultRepository();
        PageCollectionServicePtr pageCollectionSrv = repo->getPageCollectionService();
        
        // Get the plugin
        PluginLoader *loader = PluginLoader::instance();
        BinarizerPtr binarizer = loader->getPlugin<Binarizer>("labri.OtsuBinarization.0.1.1");
        
        meta_iterator<PagePtr>::IteratorPtr it = pageCollectionSrv->getPages("(H)DIBCO");
        while(it->hasNext()) {
            try{
                PagePtr page = it->next();
                QImage pageImage = page->getImage();
                QImage bin = binarizer->binarize(pageImage);
                bin.save(page->getName()+"_bin.png");
            }catch (MessageException e) {
                qCritical() << tr("Error while binarizing a page") << e.message();
            }
        }
    }
};


int main(int argc, char *argv[])
{
    QCoreApplication app(argc, argv);
    @AppName@ myApp (app);
    return myApp.exec();
}
